--[[=============================================================================
    Main script file for driver

    Copyright 2016 Control4 Corporation. All Rights Reserved.
===============================================================================]]
require "common.c4_driver_declarations"
require "common.c4_common"
require "common.c4_init"
require "common.c4_property"
require "common.c4_command"
require "common.c4_notify"
require "common.c4_utils"
require "common_modified.c4_network_connection"
require "common_modified.c4_serial_connection"
require "lib.c4_timer"
require "actions"
require "device_specific_commands"
require "device_messages"
require "proxy_init"
require "properties"
require "proxy_commands"
require "connections"

-- This macro is utilized to identify the version string of the driver template version used.
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.driver = "2016.01.08"
end

--[[=============================================================================
    Initialization Code
===============================================================================]]
function ON_DRIVER_EARLY_INIT.main()
	
end

function ON_DRIVER_INIT.main()
    -- TODO: Change the logger name
    SetLogName("Pakedge_PDU_P_Series_c4z")
    
    --Variables
    C4:AddVariable('Temperature', "0", "NUMBER", true, false)
    C4:AddVariable('Humidity', "0", "NUMBER", true, false)
    C4:AddVariable('Current A', "0", "NUMBER", true, false)
    C4:AddVariable('Current B', '0', "NUMBER", true, false)
    C4:AddVariable('5', '0', "NUMBER", true, true)  --Added these dummy varaibles to get the variable could up to 1012 to work with the temp_display driver
    C4:AddVariable('6', '0', "NUMBER", true, true)
    C4:AddVariable('7', '0', "NUMBER", true, true)
    C4:AddVariable('8', '0', "NUMBER", true, true)
    C4:AddVariable('9', '0', "NUMBER", true, true)
    C4:AddVariable('10', '0', "NUMBER", true, true)
    C4:AddVariable('11', '0', "NUMBER", true, true)
    C4:AddVariable('SendTemp', '0', "NUMBER", true, true)

    --C4:SetPropertyAttribs ("Username", 1)
    --C4:SetPropertyAttribs ("Password", 1)
    
    --Globals

    gUsername = gUsername or 'pakedge'
    gPassword = gPassword or 'pakedgep'
    gTemp = ""
    gCurra = ""
    gCurrb = ""
    gTemperature = ""
    gHumidity = ""
    gCurrentA = ""
    gCurrentB = ""
    gModelHideUpdateProperties = ""
    
    gInitTimer = c4_timer:new("InitTimer", 1, "SECONDS", OnInitTimerExpired, false)
    gPowerOutletStatusDebounceTimer = c4_timer:new("PowerOutletStatusDebounceTimer", 1, "SECONDS", OnPowerOutletStatusDebounceTimerExpired, false)
    gPollingRestartTimer = c4_timer:new("PollingRestartTimer", 2, "MINUTES", OnPollingRestartTimerExpired, false)
    gLoginTimer = c4_timer:new("LoginTimer", 1, "SECONDS", OnLoginTimerExpired, false)
    
    gOutlets = 0
    RelayToggleState = {}
    gLoginAttempts = 0
    
    Notify = {
	  [0] = "OPENED",
	  [1] = "CLOSED",
    }

    if (PersistData == nil) then -- no Persist Data table exists (newly added driver) so create empty table
				PersistData = {}
    end

    if(PersistData["OutletBinding"] == nil) then -- Add a new subtable for the OutletBinding information if one does not exists
				PersistData["OutletBinding"] = {}
    end

    if(PersistData ~= nil) then -- this is a driver that is restarting and has data that we need to restore
	   Names = PersistData["OutletBinding"] -- adds the table of outlet names
	   for k,v in pairs(PersistData["OutletBinding"]) do
		  C4:AddDynamicBinding(k, "CONTROL", true, v , "RELAY", false, false) -- recreates the dynamic binding based on the stored data

		  -- Recreate any variables you have for each outlet
		  --C4:AddVariable(v .. ' current', "0", "NUMBER", true, false)
		  --C4:AddVariable(v .. ' peak', "0", "NUMBER", true, false)
		  --C4:AddVariable(v .. ' power', "0", "NUMBER", true, false)
		  --C4:AddVariable(v .. ' alert', '0', "BOOL", true, false)

		  -- If you have alert events for outlets then these can be re-created as below
		  --C4:AddEvent(100 + k, 'Outlet ' .. k .. ' alert', 'When NAME outlet ' .. k .. ' reports an alert')
	   end

	   -- This initializes gOutlets either to be the persisted value or sets it to 0
	   gOutlets = PersistData.gOutlets or 0
	   -- This ensures that the persisted data is initialized to be the same as gOutlets
	   PersistData.gOutlets = gOutlets
    end
end


function ON_DRIVER_LATEINIT.main()
	--C4:urlSetTimeout (20)
end


--[[=============================================================================
    Driver Code
===============================================================================]]
function PackAndQueueCommand(...)
    local command_name = select(1, ...) or ""
    local command = select(2, ...) or ""
    local command_delay = select(3, ...) or tonumber(Properties["Command Delay Milliseconds"])
    local delay_units = select(4, ...) or "MILLISECONDS"
    LogTrace("PackAndQueueCommand(), command_name = " .. command_name .. ", command delay set to " .. command_delay .. " " .. delay_units)
    if (command == "") then
	   LogWarn("PackAndQueueCommand(), command_name = " .. command_name .. ", command string is empty - exiting PackAndQueueCommand()")
	   return
    end
	
	-- TODO: pack command with any any required starting or ending characters
    local cmd, stx, etx
    if (gControlMethod == "Network") then
		-- TODO: define any required starting or ending characters. 
		stx = ""
		etx = "\r"
		cmd = stx .. command .. etx
    elseif (gControlMethod == "Serial") then
		-- TODO: define any required starting or ending characters. 
		stx = ""
		etx = "\r"
		cmd = stx .. command .. etx
    elseif (gControlMethod == "IR") then
		cmd = command
    else
		LogWarn("PackAndQueueCommand(): gControlMethod is not valid, ".. gControlMethod)
		return
    end
    
    gCon:QueueCommand(cmd, command_delay, delay_units, command_name)
end

function PackAndQueuePriority1Command(...)
    local command_name = select(1, ...) or ""
    local command = select(2, ...) or ""
    local command_delay = select(3, ...) or tonumber(Properties["Command Delay Milliseconds"])
    local delay_units = select(4, ...) or "MILLISECONDS"
    LogTrace("PackAndQueuePriority1Command(), command_name = " .. command_name .. ", command delay set to " .. command_delay .. " " .. delay_units)
    if (command == "") then
	   LogWarn("PackAndQueueCommand(), command_name = " .. command_name .. ", command string is empty - exiting PackAndQueueCommand()")
	   return
    end
	
	-- TODO: pack command with any any required starting or ending characters
    local cmd, stx, etx
    if (gControlMethod == "Network") then
		-- TODO: define any required starting or ending characters. 
		stx = ""
		etx = "\r"
		cmd = stx .. command .. etx
    elseif (gControlMethod == "Serial") then
		-- TODO: define any required starting or ending characters. 
		stx = ""
		etx = "\r"
		cmd = stx .. command .. etx
    elseif (gControlMethod == "IR") then
		cmd = command
    else
		LogWarn("PackAndQueueCommand(): gControlMethod is not valid, ".. gControlMethod)
		return
    end
    
    gCon:QueuePriority1Command(cmd, command_delay, delay_units, command_name) 
end

function PackAndQueuePriority2Command(...)
    local command_name = select(1, ...) or ""
    local command = select(2, ...) or ""
    local command_delay = select(3, ...) or tonumber(Properties["Command Delay Milliseconds"])
    local delay_units = select(4, ...) or "MILLISECONDS"
    LogTrace("PackAndQueuePriority2Command(), command_name = " .. command_name .. ", command delay set to " .. command_delay .. " " .. delay_units)
    if (command == "") then
	   LogWarn("PackAndQueuePriority2Command(), command_name = " .. command_name .. ", command string is empty - exiting PackAndQueueCommand()")
	   return
    end
	
	-- TODO: pack command with any any required starting or ending characters
    local cmd, stx, etx
    if (gControlMethod == "Network") then
		-- TODO: define any required starting or ending characters. 
		stx = ""
		etx = "\r"
		cmd = stx .. command .. etx
    elseif (gControlMethod == "Serial") then
		-- TODO: define any required starting or ending characters. 
		stx = ""
		etx = "\r"
		cmd = stx .. command .. etx
    elseif (gControlMethod == "IR") then
		cmd = command
    else
		LogWarn("PackAndQueueCommand(): gControlMethod is not valid, ".. gControlMethod)
		return
    end
    
    gCon:QueuePriority2Command(cmd, command_delay, delay_units, command_name) 
end

function tonumber2(str, base)
  local s = str:gsub(",", ".") -- Assume US Locale decimal separator
  local num = tonumber(s, base)
  if (num == nil) then
    s = str:gsub("%.", ",") -- Non-US Locale decimal separator
    num = tonumber(s, base)
  end
  return num
end

